class TestMagicRunPass(tt.TempFileMixin):

    def setUp(self):
        content = "a = [1,2,3]\nb = 1"
        self.mktmp(content)

    def run_tmpfile(self):
        _ip = get_ipython()
        # This fails on Windows if self.tmpfile.name has spaces or "~" in it.
        # See below and ticket https://bugs.launchpad.net/bugs/366353
        _ip.run_line_magic("run", self.fname)

    def run_tmpfile_p(self):
        _ip = get_ipython()
        # This fails on Windows if self.tmpfile.name has spaces or "~" in it.
        # See below and ticket https://bugs.launchpad.net/bugs/366353
        _ip.run_line_magic("run", "-p %s" % self.fname)

    def test_builtins_id(self):
        """Check that %run doesn't damage __builtins__ """
        _ip = get_ipython()
        # Test that the id of __builtins__ is not modified by %run
        bid1 = id(_ip.user_ns['__builtins__'])
        self.run_tmpfile()
        bid2 = id(_ip.user_ns['__builtins__'])
        assert bid1 == bid2

    def test_builtins_type(self):
        """Check that the type of __builtins__ doesn't change with %run.

        However, the above could pass if __builtins__ was already modified to
        be a dict (it should be a module) by a previous use of %run.  So we
        also check explicitly that it really is a module:
        """
        _ip = get_ipython()
        self.run_tmpfile()
        assert type(_ip.user_ns["__builtins__"]) == type(sys)

    def test_run_profile(self):
        """Test that the option -p, which invokes the profiler, do not
        crash by invoking execfile"""
        self.run_tmpfile_p()
    def test_run_debug_twice(self):
        # https://github.com/ipython/ipython/issues/10028
        _ip = get_ipython()
        with tt.fake_input(["c"]):
            _ip.run_line_magic("run", "-d %s" % self.fname)
        with tt.fake_input(["c"]):
            _ip.run_line_magic("run", "-d %s" % self.fname)

    def test_run_debug_twice_with_breakpoint(self):
        """Make a valid python temp file."""
        _ip = get_ipython()
        with tt.fake_input(["b 2", "c", "c"]):
            _ip.run_line_magic("run", "-d %s" % self.fname)

        with tt.fake_input(["c"]):
            with tt.AssertNotPrints("KeyError"):
                _ip.run_line_magic("run", "-d %s" % self.fname)

class TestMagicRunSimple(tt.TempFileMixin):

    def test_simpledef(self):
        """Test that simple class definitions work."""
        src = ("class foo: pass\n"
               "def f(): return foo()")
        self.mktmp(src)
        _ip.run_line_magic("run", str(self.fname))
        _ip.run_cell("t = isinstance(f(), foo)")

    @pytest.mark.xfail(
        platform.python_implementation() == "PyPy",
        reason="expecting __del__ call on exit is unreliable and doesn't happen on PyPy",
    )
    def test_obj_del(self):
        """Test that object's __del__ methods are called on exit."""
        src = ("class A(object):\n"
               "    def __del__(self):\n"
               "        print('object A deleted')\n"
               "a = A()\n")
        self.mktmp(src)
        err = None

    def test_aggressive_namespace_cleanup(self):
        """Test that namespace cleanup is not too aggressive GH-238

        Returning from another run magic deletes the namespace"""

        with tt.TempFileMixin() as empty:
            empty.mktmp("")
            # On Windows, the filename will have \users in it, so we need to use the
            # repr so that the \u becomes \\u.
            src = (
                "for i in range(5):\n"
                "   try:\n"
                "       ip.magic(%r)\n"
                "       print(i)\n"
                "       break\n" % ("run " + empty.fname)
            )
            self.mktmp(src)
            _ip.run_line_magic("run", str(self.fname))
            _ip.run_cell("ip == get_ipython()")
            assert _ip.user_ns["i"] == 4

    def test_run_second(self):
        """Test that running a second file doesn't clobber the first, gh-3547"""
        self.mktmp("avar = 1\n" "def afunc():\n" "  return avar\n")

        with tt.TempFileMixin() as empty:
            empty.mktmp("")

            _ip.run_line_magic("run", self.fname)
            _ip.run_line_magic("run", empty.fname)
            assert _ip.user_ns["afunc"]() == 1

    def test_tclass(self):
        mydir = os.path.dirname(__file__)
        tc = os.path.join(mydir, "tclass")
        src = f"""\
import gc
%run "{tc}" C-first
gc.collect(0)
%run "{tc}" C-second
gc.collect(0)
%run "{tc}" C-third
gc.collect(0)
%reset -f
"""
        self.mktmp(src, ".ipy")
        out = """\
ARGV 1-: ['C-first']
ARGV 1-: ['C-second']
tclass.py: deleting object: C-first
ARGV 1-: ['C-third']
tclass.py: deleting object: C-second
tclass.py: deleting object: C-third
"""
        err = None
        tt.ipexec_validate(self.fname, out, err)

    def test_run_i_after_reset(self):
        """Check that %run -i still works after %reset (gh-693)"""
        src = "yy = zz\n"
        self.mktmp(src)
        _ip.run_cell("zz = 23")
        try:
            _ip.run_line_magic("run", "-i %s" % self.fname)
            assert _ip.user_ns["yy"] == 23
        finally:
            _ip.run_line_magic("reset", "-f")

        _ip.run_cell("zz = 23")
        try:
            _ip.run_line_magic("run", "-i %s" % self.fname)
            assert _ip.user_ns["yy"] == 23
        finally:
            _ip.run_line_magic("reset", "-f")

    def test_unicode(self):
        """Check that files in odd encodings are accepted."""
        mydir = os.path.dirname(__file__)
        na = os.path.join(mydir, "nonascii.py")
        _ip.magic('run "%s"' % na)
        assert _ip.user_ns["u"] == "Ўт№Ф"

        """Test handling of `__file__` attribute in `%run <file>.py`."""
        src = "t = __file__\n"
        self.mktmp(src)
        _missing = object()
        file1 = _ip.user_ns.get("__file__", _missing)
        _ip.run_line_magic("run", self.fname)
        file2 = _ip.user_ns.get("__file__", _missing)

        # namespace.
        assert _ip.user_ns["t"] == self.fname
        # Check that __file__ was not leaked back into user_ns.
        assert file1 == file2

    def test_run_ipy_file_attribute(self):
        """Test handling of `__file__` attribute in `%run <file.ipy>`."""
        src = "t = __file__\n"
        self.mktmp(src, ext='.ipy')
        _missing = object()
        file1 = _ip.user_ns.get("__file__", _missing)
        _ip.run_line_magic("run", self.fname)
        file2 = _ip.user_ns.get("__file__", _missing)
        # Check that __file__ was equal to the filename in the script's
        # namespace.
        assert _ip.user_ns["t"] == self.fname

        # Check that __file__ was not leaked back into user_ns.

    def test_run_formatting(self):
        """ Test that %run -t -N<N> does not raise a TypeError for N > 1."""
        src = "pass"
        self.mktmp(src)
        _ip.run_line_magic("run", "-t -N 1 %s" % self.fname)
        _ip.run_line_magic("run", "-t -N 10 %s" % self.fname)
    def test_ignore_sys_exit(self):
        src = "import sys; sys.exit(1)"
        self.mktmp(src)
        with tt.AssertPrints("SystemExit"):
            _ip.run_line_magic("run", self.fname)

        with tt.AssertNotPrints("SystemExit"):
            _ip.run_line_magic("run", "-e %s" % self.fname)

    def test_run_nb(self):
        """Test %run notebook.ipynb"""
        pytest.importorskip("nbformat")
        from nbformat import v4, writes
        nb = v4.new_notebook(
           cells=[
                v4.new_markdown_cell("The Ultimate Question of Everything"),
                v4.new_code_cell("answer=42")
            ]
        )
        src = writes(nb, version=4)
        self.mktmp(src, ext='.ipynb')



    def test_run_nb_error(self):
        """Test %run notebook.ipynb error"""
        from nbformat import v4, writes

        # %run when a file name isn't provided
        pytest.raises(Exception, _ip.magic, "run")

        # %run when a file doesn't exist
        pytest.raises(Exception, _ip.magic, "run foobar.ipynb")

        # %run on a notebook with an error
        nb = v4.new_notebook(
           cells=[
                v4.new_code_cell("0/0")
            ]
        )
        src = writes(nb, version=4)
        self.mktmp(src, ext='.ipynb')
        pytest.raises(Exception, _ip.magic, "run %s" % self.fname)

    def test_file_options(self):
        src = ('import sys\n'
               'a = " ".join(sys.argv[1:])\n')
        test_opts = "-x 3 --verbose"
        _ip.run_line_magic("run", "{0} {1}".format(self.fname, test_opts))


class TestMagicRunWithPackage(unittest.TestCase):

    def writefile(self, name, content):
        path = os.path.join(self.tempdir.name, name)
        d = os.path.dirname(path)
        if not os.path.isdir(d):
            os.makedirs(d)
        with open(path, "w", encoding="utf-8") as f:
            f.write(textwrap.dedent(content))

    def setUp(self):
        self.package = package = 'tmp{0}'.format(''.join([random.choice(string.ascii_letters) for i in range(10)]))
        """Temporary  (probably) valid python package name."""

        self.value = int(random.random() * 10000)

        self.tempdir = TemporaryDirectory()
        self.__orig_cwd = os.getcwd()
        sys.path.insert(0, self.tempdir.name)

        self.writefile(os.path.join(package, '__init__.py'), '')
        self.writefile(os.path.join(package, 'sub.py'), """
        x = {0!r}
        """.format(self.value))
        self.writefile(os.path.join(package, 'relative.py'), """
        from .sub import x
        """)
        self.writefile(os.path.join(package, 'absolute.py'), """
        from {0}.sub import x
        """.format(package))
        self.writefile(os.path.join(package, 'args.py'), """
        import sys
        a = " ".join(sys.argv[1:])
        """.format(package))